#define __RAWKS_DUMP__
#define DBG 0

#include "defines.h"
#include <stdio.h>
#include "globals.h"

void cls(void)
{
  printf("\033[0;0H\033[J");
  fflush(stdout);
}

void dump(void)
{
  cls();
  fprintf(stderr,"D_time = %f\n",D_time);
  fprintf(stderr,"Firstbullet_i = %d\n",Firstbullet_i);
  fprintf(stderr,"Nextbullet_i = %d\n",Nextbullet_i);
  fprintf(stderr,"Firstpartic_i = %d\n",Firstpartic_i);
  fprintf(stderr,"Nextpartic_i = %d\n",Nextpartic_i);
}
