#ifndef __RAWKS_CONSTANTS__
#define __RAWKS_CONSTANTS__

unsigned int mycolors[16] = {0xFF0000,0xCF3F00,0x9F6f00,0x6F9F00,0x3FCF00,
		     0x00FF00,0x00CF3F,0x009F6F,0x006F9F,0x003FCF,
		     0x0000FF,0x3F00CF,0x9F009F,0x6F006F,0xCF003F,
		     0xFF0000};

sfx_info Sfx_info[SFX_N_SOUNDS] =
{
/* name, repeat , maxcount, pri */

  "on.aiff",	0,1,SFX_PRI_BACKGROUND,
  "expl_1.aiff",	0,1,SFX_PRI_ROCK,
  "expl_2.aiff",	0,1,SFX_PRI_ROCK,
  "expl_3.aiff",	0,1,SFX_PRI_ROCK,
  "1.aiff",		0,1,SFX_PRI_BACKGROUND,
  "2.aiff",		0,1,SFX_PRI_BACKGROUND,
  "3.aiff",		0,1,SFX_PRI_BACKGROUND,
  "4.aiff",		0,1,SFX_PRI_BACKGROUND,
  "alive.aiff",		0,1,SFX_PRI_BACKGROUND,
  "fire.aiff",		0,1,SFX_PRI_FIRE,
  "bell.aiff",		0,1,SFX_PRI_BACKGROUND,
  "congrat.aiff",	0,1,SFX_PRI_BACKGROUND,
  "game_over.aiff",	0,1,SFX_PRI_BACKGROUND,
  "sad.aiff",		0,1,SFX_PRI_BACKGROUND,
  "thrust.aiff",	1,1,SFX_PRI_THRUST,
  "mystic.aiff",	0,1,SFX_PRI_BACKGROUND,
  "intro.aiff",		0,1,SFX_PRI_BACKGROUND,
  "laugh.aiff",		0,1,SFX_PRI_BACKGROUND,
};

xyz Ship1_verts[SHIP1_SIZE] = { /* Space ship */
    0.000,  0.02, 0.0 ,
   -0.010, -0.02, 0.0 ,
   -0.006, -0.015, 0.0 ,
    0.006, -0.015, 0.0 ,
    0.010, -0.02, 0.0 ,
};

xyz Ship1_thrust[3] = { /* Thrust for space ship */
   -0.004, -0.015, 0.0 ,
    0.000, -0.030, 0.0 ,
    0.004, -0.015, 0.0 ,
};

/* object_geom Ships[] = { */
/*   { SHIP1_SIZE,&Ship1_verts[0] }, /* Ship */
/*   { 3,&Ship1_thrust[0] }, /* Thrust */
/* }; */

#endif /* __RAWKS_CONSTANTS__ */
