#ifndef __RAWKS_DEFINES__
#include "defines.h"
#endif

#ifndef __RAWKS_STRUCT__
#include "struct.h"
#endif

/* I decided to take this space to tell you that I am happy */
/* just because :-) */
#include "cpr_out.h"
