#define __RAWKS_COLOR__
#define DBG 0

/* This file is made to be included with other files */

unsigned int ship_color(int player) {
  unsigned int lc[5] = {0xFFFFFF,0x77FFFF,0x77FF77,0x7777FF,0xFF7777};

  return (lc[player]);
}
