#define __RAWKS_MAIN__

#include <stdio.h>
#include <sys/time.h>
#include "defines.h"
#include "struct.h"
#include "protos.h"
#include "globals.h"
#include "constants.h"

Matrix Identity = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};

/* Shows example use of routines */
void main(int argc, char **argv)
{
  level l;
  FILE *file;

  file = fopen("../Mission/universe","r");

  while(1) {
    read_level(&l,file);
    if(!(l.rocks)) break;
    printf("\n\nCompleted reading level, \"%s\":\n\n",l.name);
    print_rdag(l.rocks);
    free_geometry(l.rocks);
  }
  fclose(file);
  exit(0);
}
