#define __RAWKS_GFX__
#define DBG 0

#include "gl.h"

/* Boy, isn't this silly?
   I'm too lazy to take this out - it turned out that this would really limit
   which machine I could run this game on if I used anything besides normal
   drawing *sigh*... I really need to learn how to comment my code... 
   - Aaron Michael Hightower */

void my_swapbuffers(int mode)
{
  swapbuffers();
}
