extern void set_frame_rate(int new_rate);
extern void copy_time(struct timeval *dst, struct timeval *src);
extern void set_time(struct timeval *dst, struct timeval *src, long secs, long usecs);
extern void add_to_time(struct timeval *dst, long secs, long usecs);
extern float pace_frame(struct timeval *buf);
extern void start_time(struct timeval *buf);
extern float end_time(struct timeval *bufThen, struct timeval *bufNow);
extern long elapsed_sec_tenths(struct timeval *bufThen, struct timeval *bufNow);
