extern void read_in_rock(rock *Rock, int force_load);
extern void load_geom_rdag(rock *rdag);
extern int filter_dotstar(struct dirent *dep);
extern int filter_dotdot(struct dirent *dep);
extern void loadgeometry(void);
extern void reload_mission(void);
extern void reload_ifnewer(void);
extern void free_all_geometry_by_name(char *name);
